@@ -8,7 +8,7 @@
 //
 // Copyright (C) 2013	Xiuwen Zheng (zhengx@u.washington.edu)
 //
-// This file is part of HIBAG package.
+// This file is part of HIBAG R package.
 //
 // HIBAG is free software: you can redistribute it and/or modify it
 // under the terms of the GNU General Public License Version 3 as
@@ -26,10 +26,10 @@
 // ===============================================================
 // Name           : LibHLA
 // Author         : Xiuwen Zheng
-// Version        : 1.2.0
+// Version        : 1.2.1
 // Copyright      : Xiuwen Zheng (GPL v3.0)
 // Created        : 11/14/2011
-// Last modified  : 07/20/2013
+// Last modified  : 10/29/2013
 // Description    : HLA Genotype Imputation with Attribute Bagging
 // ===============================================================
 
@@ -43,7 +43,7 @@
 
 #include <StructHLA.h>
 
-#ifdef HIBAG_GPU_SUPPORT
+#ifdef HIBAG_ALLOW_GPU_SUPPORT
 #  if defined(HIBAG_SYS_UNIX)
 #    include <dlfcn.h>
 #  elif defined(HIBAG_SYS_WIN)
@@ -100,7 +100,7 @@ static const TFLOAT PRUNE_RELTOL_LOGLIK = 0.1;
 
 
 
-#ifdef HIBAG_GPU_SUPPORT
+#ifdef HIBAG_ALLOW_GPU_SUPPORT
 
 // whether run in double precision, depending on the device
 static bool _gpuRunInDoublePrecision = false;
@@ -754,7 +754,7 @@ void CHaplotypeList::Print()
 }
 
 
-#ifdef HIBAG_GPU_SUPPORT
+#ifdef HIBAG_ALLOW_GPU_SUPPORT
 
 void CHaplotypeList::InitGPUHostData(int *&_HLA_HapIdx, void *&_HapList)
 {
@@ -856,7 +856,7 @@ void CHaplotypeList::InitGPUHostData(int *&_HLA_HapIdx, void *&_HapList)
 #endif
 
 
-#ifdef HIBAG_GPU_SUPPORT
+#ifdef HIBAG_ALLOW_GPU_SUPPORT
 inline void CHaplotypeList::FreqGPUHostData(int *&_HLA_HapIdx, void *&_HapList)
 {
 	// no worry if _HLA_HapIdx == NULL
@@ -1605,7 +1605,7 @@ TFLOAT CVariableSelection::_OutOfBagAccuracy(CHaplotypeList &Haplo)
 
 	int TotalCnt, CorrectCnt;
 
-#ifdef HIBAG_GPU_SUPPORT
+#ifdef HIBAG_ALLOW_GPU_SUPPORT
 	if (_gpuOutOfBagAcc_F64 || _gpuOutOfBagAcc_F32)
 	{
 		int *_HLA_HapIdx = NULL;
@@ -1647,7 +1647,7 @@ TFLOAT CVariableSelection::_OutOfBagAccuracy(CHaplotypeList &Haplo)
 			}
 		}
 
-#ifdef HIBAG_GPU_SUPPORT
+#ifdef HIBAG_ALLOW_GPU_SUPPORT
 	}
 #endif
 
@@ -1667,7 +1667,7 @@ TFLOAT CVariableSelection::_InBagLogLik(CHaplotypeList &Haplo)
 	HIBAG_CHECKING(Haplo.Num_SNP != _GenoList.Num_SNP,
 		"CVariableSelection::_InBagLogLik, Haplo and GenoList should have the same number of SNP markers.");
 
-#ifdef HIBAG_GPU_SUPPORT
+#ifdef HIBAG_ALLOW_GPU_SUPPORT
 
 #endif
 
@@ -1814,7 +1814,7 @@ void CVariableSelection::Search(CBaseSampling &VarSampling,
 	Out_Global_Max_OutOfBagAcc = Global_Max_OutOfBagAcc;
 }
 
-#ifdef HIBAG_GPU_SUPPORT
+#ifdef HIBAG_ALLOW_GPU_SUPPORT
 
 int CVariableSelection::InitGPUHostData_OutOfBag(TGPU_Genotype *&_GList)
 {
